#!/usr/bin/perl -w
use strict;
use Getopt::Long;
use File::Basename;


###############################################################
##################### DOCUMENTATION ###########################

## Author: Yun Li
## Date: 2009-09-17
## Goal: 
##       concatenate two sets of haplotypes (e.g., CEU and YRI)
##	    discard SNPs appearing only in one set
my $location = $0;
my $title = "Concatenate two sets of haplotypes V0.0.2";

print "$title\n";
print "\t\@: $location\n";
print "\tby: Yun Li(ylwtx\@umich.edu)\n\n";

## Input Parameters
##

## Input Files
## haplotype file: 
##      can have one field per line or multiple field per line; 
##      but alleles should be one field (i.e., NOT separated)
##      did not check if two sets of haplotypes are of the same format

## Output Files

################# END OF DOCUMENTATION ########################
###############################################################

my $MIN_NUM_OPTS = 4;

if($#ARGV<($MIN_NUM_OPTS*2 - 1))
{
	&usage();
	die "Failed to parse parameters\n\n";
}


my %opts = ();

	# Default Optional Options
$opts{i} = "yun.in";
$opts{o} = "catHap.out";

Getopt::Long::GetOptions(\%opts,qw(
	ah=s
	as=s
	bh=s
	bs=s
	o=s
)) || die "Failed to parse options\n\n";


&printPar();

my %aSnpsHash = (); #key: snp name, value: index, starting at zero
my $numASNPs;
my $numBSNPs;
my @indicesA = (); #indices of the overlapping SNPs in set A
my @indicesB = (); #indices of the overlapping SNPs in set B


&main();


# begin sub

sub usage
{
	print "\n";
	print "Usage:\n\t";
	print "-ah\t Haplotype file A (Required) \n\t";
	print "-as\t SNP file A (Required) \n\t";
	print "-bh\t Haplotype file B (Required) \n\t";
	print "-bs\t SNP file B (Required) \n\t";
	print "-o\t Output Prefix (Optional) \n\t";
	print "\n";
}


sub printPar
{
	print "\n";
	print "Parameters in Effect:\n";
	print "\t Haplotype file A \n\t\t-ah '$opts{ah}'\n";
	print "\t SNP file A \n\t\t-as '$opts{as}'\n";
	print "\t Haplotype file B \n\t\t-bh '$opts{bh}'\n";
	print "\t SNP file B \n\t\t-bs '$opts{bs}'\n";
	print "\t Output Prefix \n\t\t-o '$opts{o}'\n";
	print "\n";
}





sub rm
{
	if(-e $_[0])
	{
		print "WARNING: $_[0] existed and removed!!\n\n";
		system "/bin/rm -rf $_[0]";
	}
}



sub bakrm
{
	if(-e $_[0])
	{
		my $tt = $_[0].".old";
		print "WARNING: $_[0] existed and moved to $tt !!\n\n";
		system "mv $_[0] $tt";
	}
}


sub swap
{
	my $tmp = $_[0];
	$_[0] = $_[1];
	$_[1] = $tmp;
}



sub main
{
	&loadSNPs();
	&processHaplotypes();
}

sub loadSNPs
{
	#load first set of SNPs
	my $linenum = 0;
	open(IN, $opts{as}) || die "cannot open file $opts{as}\n\n";
	while (my $line = <IN>)
	{
		chomp($line);
		my @linearr=(split(/\s+/, $line));
		if (scalar(@linearr) != 1)
		{
			printf("ERROR: SNP file $opts{as} line %d (with %d fields) needs to have exact one field per line\n\n", ($linenum + 1), scalar(@linearr));
			die;
		}
		if (exists $aSnpsHash{$line})
		{
			die "ERROR: SNP $line is duplicated in $opts{as}\n\n";
		}
		$aSnpsHash{$line} = $linenum;
		$linenum++;
	}
	close(IN);
	$numASNPs = $linenum;
	printf("Read %d SNPs from $opts{as}\n", $numASNPs);
	
	# load second set of SNPs
	# did not check if a SNP is duplicated in the second set
	my $of = $opts{o}.".snps";
	&bakrm($of);
	open(OUT, ">>$of") || die "cannot create file $of\n\n";
	$linenum = 0;
	open(IN, $opts{bs}) || die "cannot open file $opts{bs}\n\n";
	while (my $line = <IN>)
	{
		chomp($line);
		my @linearr=(split(/\s+/, $line));
		if (scalar(@linearr) != 1)
		{
			printf("ERROR: SNP file $opts{bs} line %d (with %d fields) needs to have exact one field per line\n\n", ($linenum + 1), scalar(@linearr));
			die;
		}
		if (exists $aSnpsHash{$line})
		{
			if (scalar(@indicesA) != 0 && $aSnpsHash{$line} <= $indicesA[$#indicesA])
			{
				die "ERROR: SNP $line goes back in order\n\n";
			}
			
			push @indicesA, $aSnpsHash{$line};
			push @indicesB, $linenum;
			print OUT "$line\n";
		}
		$linenum++;
	}
	close(IN);
	close(OUT);
	
	$numBSNPs = $linenum;
	printf("Read %d SNPs from $opts{bs}\n", $numBSNPs);
	printf("%d SNPs overlap\n",scalar(@indicesA));
	
}


sub processHaplotypes
{
	my $of = $opts{o}.".hap";
	&bakrm($of);
	open(OUT, ">>$of") || die "cannot create file $of\n\n";
	
	# load first set of haplotypes and print only those overlapping
	open(IN, $opts{ah}) || die "cannot open file $opts{ah}\n\n";
	my $linenum = 0;
	while(my $line = <IN>)
	{
		$linenum ++;
		chomp($line);
		my @linearr = (split(/\s+/, $line));
		if (scalar(@linearr) < 1)
		{
			die "ERROR: line $linenum in $opts{ah} is empty\n\n";
		}
		my @alleles = (split(//,$linearr[$#linearr]));
		if (scalar(@alleles) != $numASNPs)
		{
			printf("ERROR: line $linenum in $opts{ah} expects to have $numASNPs alleles while observed %d\n", scalar(@alleles));
			die;
		}
		for (my $i = 0; $i < $#linearr; $i++)
		{
			print OUT "$linearr[$i] ";
		}
		
		for (my $i = 0; $i <= $#indicesA; $i++)
		{
			print OUT "$alleles[$indicesA[$i]]";
		}
		print OUT "\n";
	}
	close(IN);
	
	# load second set of haplotypes and print only those overlapping
	open(IN, $opts{bh}) || die "cannot open file $opts{bh}\n\n";
	$linenum = 0;
	while(my $line = <IN>)
	{
		$linenum ++;
		chomp($line);
		my @linearr = (split(/\s+/, $line));
		if (scalar(@linearr) < 1)
		{
			die "ERROR: line $linenum in $opts{bh} is empty\n\n";
		}
		my @alleles = (split(//,$linearr[$#linearr]));
		if (scalar(@alleles) != $numBSNPs)
		{
			printf("ERROR: line $linenum in $opts{bh} expects to have $numBSNPs alleles while observed %d\n", scalar(@alleles));
			die;
		}
		for (my $i = 0; $i < $#linearr; $i++)
		{
			print OUT "$linearr[$i] ";
		}
		
		for (my $i = 0; $i <= $#indicesB; $i++)
		{
			print OUT "$alleles[$indicesB[$i]]";
		}
		print OUT "\n";
	}
	close(IN);
	
	close(OUT);
	
}



