#!/usr/bin/perl -w
use strict;
use Getopt::Long;
use File::Basename;


###############################################################
##################### DOCUMENTATION ###########################

## Author: Yun Li
## Date: 2009-06-23
## Goal: Extract haplotypes for a subset of SNPs
##       $opts{r}: can contain markers not in $opts{s}, will simply be ignored
##		 : can be of any order, output will be in the original $opts{s} order
##                 
my $location = $0;
my $title = "h2h.pl Version 0.0.3";

print "$title\n";
print "\t\@: $location\n";
print "\tby: Yun Li(ylwtx\@umich.edu)\n\n";

## Input Parameters
##

## Input Files
##

## Output Files
## (1) .hap
## (2) .snps

################# END OF DOCUMENTATION ########################
###############################################################

my $MIN_NUM_OPTS = 4;

if($#ARGV<($MIN_NUM_OPTS*2 - 1))
{
	&usage();
	die "Failed to parse parameters\n\n";
}


my %opts = ();

	# Default Optional Options
$opts{col} = 3;
$opts{o} = "h2h.out";

Getopt::Long::GetOptions(\%opts,qw(
	h=s
	s=s
	r=s
	col=i
	o=s
)) || die "Failed to parse options\n\n";


&printPar();

my $numsnps = 0;
my %snps = ();   # key:   SNP name;
                 # value: SNP index in $opts{s} (0,1,2,...);
my %rsnps = ();  # key:   SNP index in $opts{s} (0,1,2,...);
                 # value: SNP name;
my @indices = (); 
&main();


# begin sub

sub usage
{
	print "\n";
	print "Usage:\n\t";
	print "-h\t Haplotype File (Required) \n\t";
	print "-s\t SNP List File (Required) \n\t";
	print "-r\t Relevant/Desired SNP List File (Required) \n\t";
	print "-o\t Output Haplotype File (Required) \n\t";
	print "-col\t Haplotype Startint Column (Optional; default = 3) \n\t";
	print "\n";
}


sub printPar
{
	print "\n";
	print "Parameters in Effect:\n";
	print "\t Haplotype File \n\t\t-h '$opts{h}'\n";
	print "\t SNP List File \n\t\t-s '$opts{s}'\n";
	print "\t Relevant/Desired SNP List File \n\t\t-r '$opts{r}'\n";
	print "\t Output Haplotype File \n\t\t-o '$opts{o}'\n";
	print "\t Haplotype Startint Column \n\t\t-col '$opts{col}'\n";
	print "\n";
}





sub rm
{
	if(-e $_[0])
	{
		print "WARNING: $_[0] existed and removed!!\n\n";
		system "/bin/rm -rf $_[0]";
	}
}



sub bakrm
{
	if(-e $_[0])
	{
		my $tt = $_[0].".old";
		print "WARNING: $_[0] existed and moved to $tt !!\n\n";
		system "mv $_[0] $tt";
	}
}


sub swap
{
	my $tmp = $_[0];
	$_[0] = $_[1];
	$_[1] = $tmp;
}


sub main
{
  &loadsnps();
  &loadRsnps();
  &processHaplotypes();
}



sub loadsnps
{
  open(IN, $opts{s}) || die "Can't open file $opts{s}\n\n";
  my $linenum = 0;
  while(my $line = <IN>)
  {
    chomp($line);
    my @linearr = (split(/\s+/,$line));
    if ($#linearr != 0)
    {
      my $ln = ($linenum + 1);
      print "each line in $opts{s} should have exactly one field\n";
      print "while line $ln : $line\n\n";
      die;
    }
    if (exists $snps{$line})
    {
      print "SNP $line is duplicated\n\n";
      die;
    }
    $snps{$line} = $linenum;
    $linenum ++;
  }

  close(IN);
  $numsnps = $linenum;
  print "Read $numsnps SNPs from $opts{s}\n";
}

sub loadRsnps
{
  open(IN, $opts{r}) || die "Can't open file $opts{r}\n\n";
  my $linenum = 0;
  while(my $line = <IN>)
  {
    $linenum ++;
    chomp($line);
    my @linearr = (split(/\s+/,$line));
    if ($#linearr != 0)
    {
      print "each line in $opts{r} should have exactly one field\n";
      print "while line $linenum : $line\n\n";
      die;
    }
    
    if (!exists $snps{$line})
    {
      print "WARNING: ignored SNP $line b/c not in haplotype file\n";
      next;
    }
    if (exists $rsnps{$snps{$line}})
    {
      print "WARNING: SNP $line is duplicated\n\n";
      next;
    }
    
    $rsnps{$snps{$line}} = $line;
    
  }
  close(IN);

  my $outsnpf = $opts{o}.".snps";
  &bakrm($outsnpf);
  open(OUT, ">>$outsnpf") || die "cannot create file $outsnpf\n\n";
  foreach my $c (sort {$a <=> $b} keys %rsnps)
  {
    push @indices, $c;
    print OUT "$rsnps{$c}\n";
  }
  close(OUT);
  
  #for (my $i = 0; $i <= $#indices; $i++)
  #{
    #print "$indices[$i]\n";
  #}

}


sub processHaplotypes
{
  
  my $outhapf = $opts{o}.".hap";
  &rm($outhapf);
  open(OUT, ">>$outhapf") || die "cannot create file $outhapf\n\n";
  if ($opts{h} =~ /(.+)\.gz$/)
  {
    $opts{h} = $1;         
  } # Remove gz suffix if provided
  if (-r "$opts{h}.gz")
  {
    $opts{h} = "$opts{h}.gz";
    open(IN, "gunzip -c $opts{h} |") || die "Unable to open compressed file '$opts{h}': $!\n";       
  }
  else
  {
    open(IN, $opts{h}) || die "Unable to open file '$opts{h}': $!\n";
  }           

  my $linenum = 0;
  while(my $line = <IN>)
  {
    $linenum ++;
    chomp($line);
    my @linearr = (split(/\s+/, $line));
    if (scalar(@linearr) != $opts{col})
    {
      print "each line in $opts{h} should have $opts{col} fields\n";
      print "while line $linenum does not\n\n";
      die;
    }
    my @alleles = (split(//,$linearr[$#linearr]));
    if (scalar(@alleles) != $numsnps)
    {
      print "each haplotype in $opts{h} should have $numsnps alleles\n";
      print "while line $linenum does not\n\n";
      die;
    }
    for (my $i = 0; $i < ($opts{col} - 1); $i ++)
    {
      print OUT "$linearr[$i] ";
    }
    for (my $i = 0; $i <= $#indices; $i ++)
    {
      print OUT "$alleles[$indices[$i]]";
    }
    print OUT "\n";
  }
  
  close(IN);
  close(OUT);
}


